@@ -33,10 +33,8 @@
 
 using namespace Powersave::Globals;
 
-EventManagement::EventManagement()
-{
-	id_count = 0;
-}
+EventManagement::EventManagement() : id_count(0)
+{ }
 
 EventManagement::~EventManagement()
 {
@@ -70,14 +68,14 @@ bool EventManagement::remove(int id)
 
 		if (id == it->ID()) {
 			Events.erase(it);
-			return 1;
+			return true;
 		}
 	}
 
-	return 0;
+	return false;
 }
 
-int EventManagement::count()
+int EventManagement::count() const
 {
 	return Events.size();
 }
@@ -92,7 +90,7 @@ const string EventManagement::name(int id)
 		}
 	}
 
-	return "";
+	return string();
 }
 
 const string EventManagement::cur_prog(int id)
@@ -103,12 +101,12 @@ const string EventManagement::cur_prog(int id)
 			if (it->_current_program_2_execute != it->end()) {
 				return *(it->_current_program_2_execute);
 			} else {
-				return "";
+				return string();
 			}
 		}
 	}
 
-	return "";
+	return string();
 }
 
 int EventManagement::moveOn(int id)
@@ -216,7 +214,7 @@ void EventManagement::checkScriptReturn(int id, int returnValue, string &comment
 		DBus_Server::emitSignal("Progress", comment.c_str());
 		break;
 		// script has failed
-	case 1:
+	case 1: {
 		const string event_name = this->name(id);
 		pDebug(DBG_INFO, "SCRIPT %s failed to execute", event_name.c_str());
 
@@ -231,6 +229,10 @@ void EventManagement::checkScriptReturn(int id, int returnValue, string &comment
 		// remove the event from the list of pending events
 		this->remove(id);
 		break;
+		}
+	default: 
+		pDebug(DBG_ERR, "Internal error: checkScriptReturn got an unknown return value of %d", returnValue);
+		break;
 	}
 }
 
