@@ -68,7 +68,7 @@ public:
 	/** @brief get number of pending events 
 	 * @return number of still running/pending events 
 	 */
-	int count();
+	int count() const;
 
 	/** @brief removes one event and frees its memory
 	 * @param id event id
