@@ -23,6 +23,9 @@
  ***************************************************************************/
 
 #include <signal.h>
+#include <sched.h>
+#include <errno.h>
+#include <string.h>
 
 #include "main_loop.h"
 #include "powerlib.h"
@@ -34,18 +37,13 @@
 using namespace Powersave::Globals;
 
 /**** class Callback ****/
-Callback::Callback()
+Callback::Callback() : interval(-1), function(NULL), id(-1)
 {
-	id = -1;
-	interval = -1;
-	function = NULL;
 }
 
-Callback::Callback(int interval_, gboolean (*function_)(gpointer data))
+Callback::Callback(int interval_, gboolean (*function_)(gpointer data)) : 
+	interval(interval_), function(function_), id(-1)
 {
-	id = -1;
-	interval = interval_;
-	function = function_;
 }
 
 /**** class MainLoop ****/
@@ -77,8 +75,12 @@ MainLoop::MainLoop() {
 	}
 
 	// wait about 10 minutes for hal to appear
-	int waiting = 0;
+	unsigned int waiting = 0;
 	while (!DBus_Server::serviceHasOwner("org.freedesktop.Hal")) {
+		/* give other processes a better chance to evolve */
+		if (sched_yield() != 0) {
+			pDebug(DBG_ERR, "Error while yielding scheduler: %s", strerror(errno));
+		}
 		sleep(3);
 		waiting++;
 		if (waiting > 200) {
@@ -152,7 +154,7 @@ gboolean MainLoop::hwEvent_connect()
 
 gboolean MainLoop::checkThrottling_callback(gpointer data)
 {
-	if (config_obj->current_scheme->ALLOW_THROTTLING) {
+	if (config_obj->current_scheme->ALLOW_THROTTLING && !cpufreq->isSupported()) {
 		pm->checkThrottling();
 		return true;
 	}
@@ -206,3 +208,4 @@ int MainLoop::addCallback(int interval, gboolean (*function)(gpointer data))
 {
 	return g_timeout_add(interval, function, NULL);
 }
+
