@@ -32,6 +32,8 @@
 #include "clientConnection.h"
 #include "cpufreq_management.h"
 #include "globals.h"
+#include "pm_interface.h"
+#include "cpu.h"
 
 using namespace Powersave::Globals;
 
@@ -94,6 +96,9 @@ PM_Interface::~PM_Interface()
 	delete(_cC);
 	delete(_eM);
 	delete(_brightness);
+
+	// enable all CPUs
+	CPU::enableAll();
 }
 
 int PM_Interface::start()
@@ -460,16 +465,6 @@ void PM_Interface::sleep_wrapper(const string &event)
 	static CPUFREQ_MODE cpufreq_mode;
 
 	if (before_sleep) {
-		if (event == "suspend2disk") {
-			if (config_obj->current_scheme->SUSPEND2DISK_DELAY > 0)
-				sleep(config_obj->current_scheme->SUSPEND2DISK_DELAY);
-		} else if (event == "suspend2ram") {
-			if (config_obj->current_scheme->SUSPEND2RAM_DELAY > 0)
-				sleep(config_obj->current_scheme->SUSPEND2RAM_DELAY);
-		} else if (event == "standby") {
-			if (config_obj->current_scheme->STANDBY_DELAY > 0)
-				sleep(config_obj->current_scheme->STANDBY_DELAY);
-		}
 		ignore_buttons(true);
 
 		if (cpufreq->isSupported()) {
@@ -510,8 +505,8 @@ void PM_Interface::suspendNetworkManager()
 		return;
 	}
 
-	DBus_Server::sendMethodCall(DBUS_NM_SERVICE, DBUS_NM_PATH,
-				    DBUS_NM_INTERFACE, "sleep");
+	DBus_Server::sendMethodCall(DBUS_NM_INTERFACE, DBUS_NM_PATH,
+				    DBUS_NM_SERVICE, "sleep", DBUS_TYPE_INVALID);
 }
 
 void PM_Interface::resumeNetworkManager()
@@ -521,8 +516,8 @@ void PM_Interface::resumeNetworkManager()
 		return;
 	}
 
-	DBus_Server::sendMethodCall(DBUS_NM_SERVICE, DBUS_NM_PATH,
-				    DBUS_NM_INTERFACE, "wake");
+	DBus_Server::sendMethodCall(DBUS_NM_INTERFACE, DBUS_NM_PATH,
+				    DBUS_NM_SERVICE, "wake", DBUS_TYPE_INVALID);
 }
 
 int PM_Interface::executeScript(const string &script, const string &param)
@@ -638,7 +633,8 @@ void PM_Interface::ignore_buttons(bool ignore)
 
 bool PM_Interface::haveSuspend2()
 {
-	if (access("/proc/suspend2", F_OK) < 0) {
+	if (access("/proc/suspend2", F_OK) < 0
+		&& access("/sys/power/suspend2", F_OK) < 0) {
                 return false;
 	}
 
@@ -688,8 +684,11 @@ DBUS_ERROR PM_Interface::handleManagerMessage(DBusMessage * msg)
 		int capabilities;
 		/* add client. No 'request string/modes' check so far */
 		if (!dbusGetMessageInteger(msg, &capabilities, 0)) {
-			_cC->addClient(sender.c_str(), capabilities);
-			return REPLY_SUCCESS;
+			if (_cC->addClient(sender.c_str(), capabilities)) {
+				pDebug(DBG_DIAG, "added Client: %s", sender.c_str());
+				return REPLY_SUCCESS;
+			} else
+				return REPLY_INVALID_PARAM;
 		} else
 			return REPLY_INVALID_PARAM;
 	} else if (msg_member == "Disconnect") {
