@@ -48,10 +48,8 @@ typedef struct PM_STATE {
 	// THERM_MODE comes from library libpower.h
 	THERM_MODE TEMP_STATE;
 	AC_ADAPTER_STATE AC_STATE;
-	 PM_STATE() {
-		TEMP_STATE = OK;
-		AC_STATE = AC_UNKNOWN;
-}} PM_STATE;
+	 PM_STATE():TEMP_STATE(OK),AC_STATE(AC_UNKNOWN) {}
+} PM_STATE;
 
 /** sleep states requested by user, only used for APM standby/suspend
    workarounds. This should go somehow to APM_Interface later */
