@@ -50,49 +50,6 @@ using namespace Powersave::Globals;
 /** @brief the maximum length of a param including paths */
 #define MAX_PARAM_LENGTH 128
 
-#ifdef CPUFREQ_MEASURE
-
-void RT_START_handler(int)
-{
-	FILE *logFile = fopen(MEASURE_LOG_FILE, "a+");
-	if (!logFile) {
-		pDebug(DBG_ERR, "Could not open log file %s", MEASURE_LOG_FILE);
-		exit(1);
-	}
-	if (cpufreq->isSupported()) {
-		fprintf(logFile, "\nSTART ---------------------\n");
-		if (pm) {
-			fprintf(logFile, "\nVARIABLES:\nPolling Interval: %lu ms, Consider Nice: %s\n"
-				"Jump directly to highest speed on %d percent CPU load difference\n\n",
-				config_obj->POLL_INTERVAL, (config_obj->CONSIDER_NICE ? "yes" : "no"),
-				config_obj->JUMP_CPU_FREQ_MAX_LIMIT);
-		}
-		fclose(logFile);
-		cpufreq->startMeasures();
-	} else {
-		fprintf(logFile, "No CPU frequency support.\n");
-		fclose(logFile);
-	}
-}
-
-void RT_STOP_handler(int)
-{
-
-	FILE *logFile = fopen(MEASURE_LOG_FILE, "a+");
-	if (!logFile) {
-		pDebug(DBG_ERR, "Could not open log file %s", MEASURE_LOG_FILE);
-		exit(1);
-	}
-
-	if (cpufreq->isSupported()) {
-		cpufreq->stopMeasures();
-		fprintf(logFile, "STOP ---------------------\n");
-	} else
-		fprintf(logFile, "No CPU frequency support.\n");
-	fclose(logFile);
-}
-
-#endif
 
 // I currently destroy the object I am still working on.
 // Don't know yet if this is proper but should free all resources
@@ -395,15 +352,6 @@ int main(int argc, char **argv)
 	// this is for restarting select
 	signal_action.sa_flags = SA_RESTART || SA_NOCLDSTOP;
 
-#ifdef CPUFREQ_MEASURE
-	sigaddset(&signal_action.sa_mask, SIGRTMIN + 1);
-	sigaddset(&signal_action.sa_mask, SIGRTMIN + 2);
-	signal_action.sa_handler = RT_START_handler;
-	sigaction(SIGRTMIN + 1, &signal_action, 0);
-	signal_action.sa_handler = RT_STOP_handler;
-	sigaction(SIGRTMIN + 2, &signal_action, 0);
-#endif
-
 	//do a broken pipe from clients
 	signal_action.sa_handler = SIG_IGN;
 	sigaction(SIGPIPE, &signal_action, 0);
@@ -455,6 +403,9 @@ int main(int argc, char **argv)
 
 exit_failure:
 	delete pm;
+	delete cpufreq;
+	delete config_obj;
+	delete main_loop;
 	unlink(POWERSAVED_PID_FILE_PATH);
 	exit(EXIT_FAILURE);
 }
