@@ -30,6 +30,7 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <math.h>
+#include <string.h>
 
 #define ACPI_BATTERY_DIR  "/proc/acpi/battery"
 #define ACPI_AC_DIR "/proc/acpi/ac_adapter"
@@ -43,45 +44,33 @@ int getBatteriesInfo(BatteryGeneral *battery_info)
 	int last_full_capacity_sum = 0;
 	int remaining_capacity_sum = 0;
 
+	if (battery_info == NULL) {
+                errno = EINVAL;
+                return -3;
+        }
+	
 	if (!ps_hal_init()) {
 		return HAL_ERROR;
 	}
 
- 	if (battery_info == NULL) {
-		errno = EINVAL;
-		return -3;
-	}
-
 	battery_info->remaining_minutes = UNKNOWN;
 	battery_info->remaining_percent = UNKNOWN;
 	battery_info->charging_state = CHARG_STATE_UNKNOWN;
 
-	char **batteries;
-	int numBatteries;
-	batteries = libhal_find_device_by_capability(hal_ctx, "battery",
-						     &numBatteries, &dbus_error);
-
-	if (dbus_error_is_set(&dbus_error)) {
-		pDebug(DBG_INFO, "Could not get list of batteries in system: %s",
-		       dbus_error.message);
-		ps_hal_free();
-		dbus_error_free(&dbus_error);
-		libhal_free_string_array(batteries);
-		return HAL_ERROR;
-	}
+	int num_primary_batteries = numBatteries();
 
 	/* if there are no batteries */
-	if (numBatteries == 0) {
-		libhal_free_string_array(batteries);
+	if (num_primary_batteries == 0) {
 		return NO_DEVICE_ERROR;
 	}
 
 	int valid_batteries = 0;
 	int rate = 0;
-	for (int i = 0; i < numBatteries; ++i) {
-		BatteryGeneral bg;
+	BatteryGeneral bg; 
+	BatteryDetailed bd;
+	for (int i = 0; i < num_primary_batteries; ++i) {
 
-		if (getBatteryInfo(i, &bg) <= 0) {
+		if (getBatteryInfo(i, &bg) < 1) {
 			continue;
 		}
 
@@ -97,9 +86,6 @@ int getBatteriesInfo(BatteryGeneral *battery_info)
 		/* calc charging state */
 		battery_info->charging_state |= bg.charging_state;
 
-		/* calc remaining percent */
-		BatteryDetailed bd;
-
 		if (getBatteryDetailedInfo(i, &bd) < 0) {
 			continue;
 		}
@@ -147,59 +133,78 @@ int getBatteriesInfo(BatteryGeneral *battery_info)
 
 int getBatteryInfo(const int no, BatteryGeneral *bg)
 {
- 	char *value;
+ 	char *value = NULL;
+	int batt_no = -1; // will later hold the native batt.number of param no
+	char found_native_battery = 0; // true if mapping could find corresponding native battery
 
+        if (bg == NULL) {
+                errno = EINVAL;
+                return -3;
+        }
+	
 	if (!ps_hal_init()) {
 		return HAL_ERROR;
 	}
 
- 	if (bg == NULL) {
-		errno = EINVAL;
-		return -3;
-	}
+	*bg = (struct BatteryGeneral) {
+		UNKNOWN, 		//remaining_percent
+		UNKNOWN, 		//remaining_minutes
+		CHARG_STATE_UNKNOWN	//charging_state
+	};
 
-	bg->remaining_minutes = UNKNOWN;
-	bg->remaining_percent = UNKNOWN;
-	bg->charging_state = CHARG_STATE_UNKNOWN;
-
-	char **batteries;
-	int numBatteries;
+	char **batteries = NULL;
+	int num_native_batteries = 0;
 	batteries = libhal_find_device_by_capability(hal_ctx, "battery",
-						     &numBatteries, &dbus_error);
+						     &num_native_batteries, &dbus_error);
 
-	if (dbus_error_is_set(&dbus_error)) {
-		pDebug(DBG_INFO, "Could not get list of batteries in system: %s",
-		       dbus_error.message);
-		ps_hal_free();
-		dbus_error_free(&dbus_error);
-		libhal_free_string_array(batteries);
-		return HAL_ERROR;
-	}
+        if (dbus_error_is_set(&dbus_error)) {
+                pDebug(DBG_INFO, "Could not get list of batteries in system: %s",
+                       dbus_error.message);
+                ps_hal_free();
+                dbus_error_free(&dbus_error);
+                libhal_free_string_array(batteries);
+                return HAL_ERROR;
+        }
+	
+        /* if battery does not exist */
+        if (no > num_native_batteries - 1 || no < 0) {
+                libhal_free_string_array(batteries);
+                return -2;
+        }
 
 	/* if there are no batteries */
-	if (numBatteries == 0) {
-		libhal_free_string_array(batteries);
-		return NO_DEVICE_ERROR;
-	}
-
-	/* if battery does not exist */
-	if (no > numBatteries - 1 || no < 0) {
-		libhal_free_string_array(batteries);
-		return -2;
-	}
-
- 	/* check if found battery is a primary source (e.g. no ups) */
-	value = ps_hal_get_property_string(batteries[no], "battery.type");
-	if (value != NULL) {	
-		if (strcmp(value, "primary") != 0) {
-			libhal_free_string_array(batteries);
-			libhal_free_string(value);
-			return NO_DEVICE_ERROR;
+        if (num_native_batteries == 0) {
+                libhal_free_string_array(batteries);
+                return NO_DEVICE_ERROR;
+        }
+	
+	/* find the native batterynumber of (primary) batterynumber no*/
+	for (int i = 0; i < num_native_batteries; ++i) {
+		/* check if found battery is a primary source (e.g. no ups) */
+		value = ps_hal_get_property_string(batteries[i], "battery.type");
+		if (value != NULL) {
+			if (strcmp(value, "primary") == 0) {
+				batt_no++;
+				if (batt_no == no) {
+					batt_no = i;
+					found_native_battery = 1;
+					libhal_free_string(value);
+					value = NULL;
+					break;
+				}
+			}
 		}
+		libhal_free_string(value);
+		value = NULL;
 	}
-	libhal_free_string(value);
 
-	int present = ps_hal_get_property_bool(batteries[no],
+	/* check if native battery could be found */
+	if (!found_native_battery) {
+			libhal_free_string_array(batteries);
+			return NO_DEVICE_ERROR;
+	}
+
+	int present = ps_hal_get_property_bool(batteries[batt_no],
 					       "battery.present");
 	if (present < 0) {
 		return HAL_ERROR;
@@ -207,36 +212,33 @@ int getBatteryInfo(const int no, BatteryGeneral *bg)
 	else if (!present) {
 		return 0;
 	}
-	
+
 	/* get remaining percent */
-	bg->remaining_percent = ps_hal_get_property_int(batteries[no],
+	bg->remaining_percent = ps_hal_get_property_int(batteries[batt_no],
 							"battery.charge_level.percentage");
 	if (bg->remaining_percent < 0) {
 		bg->remaining_percent = UNKNOWN;
 	}
 
 	/* get charging state */
-	int state = ps_hal_get_property_bool(batteries[no], "battery.rechargeable.is_charging");
-	if (state) {
+	if (ps_hal_get_property_bool(batteries[batt_no], "battery.rechargeable.is_charging")) {
 		bg->charging_state |= CHARG_STATE_CHARGING;
-	}
 
-	state = ps_hal_get_property_bool(batteries[no], "battery.rechargeable.is_discharging");
-	if (state) {
+	        /* get remaining time */
+	        int time = ps_hal_get_property_int(batteries[batt_no], "battery.remaining_time");
+	        if (time >= 0) 
+                        bg->remaining_minutes = time / 60;
+		
+	} else if (ps_hal_get_property_bool(batteries[batt_no], "battery.rechargeable.is_discharging")) {
 		bg->charging_state |= CHARG_STATE_DISCHARGING;
+
+		/* get remaining time */
+                int time = ps_hal_get_property_int(batteries[batt_no], "battery.remaining_time");
+                if (time >= 0)
+                        bg->remaining_minutes = time / 60;
 	}
 
-	/* get remaining time */
-       	bg->remaining_minutes = UNKNOWN;
-	int time = ps_hal_get_property_int(batteries[no], "battery.remaining_time");
-
-	if (time >= 0) {
-		time = time / 60;
-
-		if (bg->charging_state != CHARG_STATE_CHARG_DISCHARG) {
-			bg->remaining_minutes = time;
-		}
-	}
+	libhal_free_string_array(batteries);
 
 	return 1;
 }
@@ -249,48 +251,71 @@ int numBatteries(void)
 	}
 
 	char **batteries;
-	int numBatteries;
+	int num_cur_native_batteries = 0;
+	static int num_native_batteries = 0; /* do some "caching" here */
+	static int num_primary_batteries = 0;
+	char *value = NULL;
 	batteries = libhal_find_device_by_capability(hal_ctx, "battery",
-						     &numBatteries, &dbus_error);
+						     &num_cur_native_batteries, &dbus_error);
+
+	if (dbus_error_is_set(&dbus_error)) {
+                pDebug(DBG_INFO, "Could not get list of batteries in system: %s",
+                       dbus_error.message);
+                ps_hal_free();
+                dbus_error_free(&dbus_error);
+                libhal_free_string_array(batteries);
+                return HAL_ERROR;
+        }
+
+	if (num_cur_native_batteries != num_native_batteries) {
+		num_native_batteries = num_cur_native_batteries;
+		num_primary_batteries = 0;
+		
+		for (int i = 0; i < num_native_batteries; ++i) {
+			/* check if found battery is a primary source (e.g. no ups) */
+			value = ps_hal_get_property_string(batteries[i], "battery.type");
+			if (value != NULL) {
+				if (!strcmp(value, "primary")) {
+					num_primary_batteries++;
+				}
+			}
+			libhal_free_string(value);
+			value = NULL;
+		}
+	} 
 
 	libhal_free_string_array(batteries);
 
-	if (dbus_error_is_set(&dbus_error)) {
-		pDebug(DBG_INFO, "Could not get list of batteries in system: %s",
-		       dbus_error.message);
-		ps_hal_free();
-		dbus_error_free(&dbus_error);
-		libhal_free_string_array(batteries);
-		return HAL_ERROR;
-	}
-
-	
-	return numBatteries;
+	return num_primary_batteries;
 }
 
 int getBatteryDetailedInfo(const int no, BatteryDetailed * bd)
 {
- 	char *value;
-
-	if (!ps_hal_init()) {
-		/* error message already thrown in initHal() */
-		return HAL_ERROR;
-	}
+ 	char *value = NULL;
+	int batt_no = -1; /* will hold the native batt.number of primary batt.number no*/
+	char found_native_battery = 0; /* will be true if mapping found corresponding native battery */
+	int chkACPI;
 
  	if (bd == NULL) {
 		errno = EINVAL;
 		return -3;
 	}
 
-	if (checkACPI() != ACPI && checkACPI() != APM) {
-		return NO_ACPI_ERROR;
-	}
-
 	if (no > MAX_BATTERIES - 1 || no < 0) {
 		return -2;
 	}
 
-	if (checkACPI() == ACPI) {
+	if (!ps_hal_init()) {
+                /* error message already thrown in initHal() */
+                return HAL_ERROR;
+        }
+
+	chkACPI = checkACPI();
+	if (chkACPI != ACPI && chkACPI != APM) {
+                return NO_ACPI_ERROR;
+        }
+	
+	if (chkACPI == ACPI) {
 		/* check whether module has been loaded and device is available */
 		int res = check_ACPI_dir(ACPI_BATTERY_DIR);
 		if (res < 0)
@@ -298,10 +323,10 @@ int getBatteryDetailedInfo(const int no, BatteryDetailed * bd)
 			return res;
 	}
 
-	char **batteries;
-	int numBatteries;
+	char **batteries = NULL;
+	int num_native_batteries = 0;
 	batteries = libhal_find_device_by_capability(hal_ctx, "battery",
-						     &numBatteries, &dbus_error);
+						     &num_native_batteries, &dbus_error);
 
 	if (dbus_error_is_set(&dbus_error)) {
 		pDebug(DBG_INFO, "Could not get list of batteries in system: %s",
@@ -313,29 +338,45 @@ int getBatteryDetailedInfo(const int no, BatteryDetailed * bd)
 	}
 
 	/* if there are no batteries */
-	if (numBatteries == 0) {
+	if (num_native_batteries == 0) {
 		libhal_free_string_array(batteries);
 		return NO_DEVICE_ERROR;
 	}
 
 	/* if battery does not exist */
-	if (no > numBatteries - 1 || no < 0) {
+	if (no > num_native_batteries - 1 || no < 0) {
 		libhal_free_string_array(batteries);
 		return -2;
 	}
 
- 	/* check if found battery is a primary source (e.g. no ups) */
-	value = ps_hal_get_property_string(batteries[no], "battery.type");
-	if (value != NULL) {	
-		if (strcmp(value, "primary") != 0) {
-			libhal_free_string_array(batteries);
-			libhal_free_string(value);
-			return NO_DEVICE_ERROR;
-		}
-	}
-	libhal_free_string(value);
+        /* find the native batterynumber of (primary) batterynumber no*/
+        for (int i = 0; i < num_native_batteries; ++i) {
+                /* check if found battery is a primary source (e.g. no ups) */
+                value = ps_hal_get_property_string(batteries[i], "battery.type");
+                if (value != NULL) {
+                        if (strcmp(value, "primary") == 0) {
+                                batt_no++;
+                                if (batt_no == no) {
+                                        batt_no = i;
+					found_native_battery = 1;
+                                        libhal_free_string(value);
+					value = NULL;
+                                        break;
+                                }
+                        }
+                }
+                libhal_free_string(value);
+		value = NULL;
+        }
 
-	int present = ps_hal_get_property_bool(batteries[no],
+        /* check if native battery could be found */
+        if (!found_native_battery) {
+                libhal_free_string_array(batteries);
+                return NO_DEVICE_ERROR;
+        }
+		
+	
+	int present = ps_hal_get_property_bool(batteries[batt_no],
 					       "battery.present");
 
 	if (present < 0)
@@ -365,14 +406,14 @@ int getBatteryDetailedInfo(const int no, BatteryDetailed * bd)
 			strcpy(bd->model_number, "UNKNOWN");
 			strcpy(bd->battery_type, "UNKNOWN");
 			bd->alarm_limit = UNKNOWN;
-			pDebug(DBG_INFO, "Battery %s not present", batteries[no]);
+			pDebug(DBG_INFO, "Battery %s not present", batteries[batt_no]);
 			libhal_free_string_array(batteries);
 			return 0;
 		}
 	}
 
 	/* capacity state */
-	value = ps_hal_get_property_string(batteries[no],
+	value = ps_hal_get_property_string(batteries[batt_no],
 					   "battery.charge_level.capacity_state");
 	if (value != NULL) {
 		if (!strcmp(value, "ok"))
@@ -386,7 +427,7 @@ int getBatteryDetailedInfo(const int no, BatteryDetailed * bd)
 	}
 
 	/* --- get charging state --- */
-	bd->charging_state = ps_hal_get_property_bool(batteries[no],
+	bd->charging_state = ps_hal_get_property_bool(batteries[batt_no],
 						      "battery.rechargeable.is_charging");
 
 	if (bd->charging_state == 1)
@@ -397,20 +438,20 @@ int getBatteryDetailedInfo(const int no, BatteryDetailed * bd)
 		bd->charging_state = CHARG_STATE_UNKNOWN;
 
 	/* present_rate */
-	bd->present_rate = ps_hal_get_property_int(batteries[no], "battery.charge_level.rate");
+	bd->present_rate = ps_hal_get_property_int(batteries[batt_no], "battery.charge_level.rate");
 	if (bd->present_rate <= 0) {
 		bd->present_rate = UNKNOWN;
 	}
 
 	/* --- get remaining capacity --- */
-	bd->remaining_capacity = ps_hal_get_property_int(batteries[no],
+	bd->remaining_capacity = ps_hal_get_property_int(batteries[batt_no],
 							 "battery.charge_level.current");
 	if (bd->remaining_capacity < 0) {
 		bd->remaining_capacity = UNKNOWN;
 	}
 
 	/* get power unit */
-	value = ps_hal_get_property_string(batteries[no], "battery.charge_level.unit");
+	value = ps_hal_get_property_string(batteries[batt_no], "battery.charge_level.unit");
 	if (value != NULL) {
 		if (!strcmp(value, "mWh")) {
 			bd->power_unit = WATT_H;
@@ -430,21 +471,22 @@ int getBatteryDetailedInfo(const int no, BatteryDetailed * bd)
 	}
 
 	/* design_capacity */
-	bd->design_capacity = ps_hal_get_property_int(batteries[no],
+	bd->design_capacity = ps_hal_get_property_int(batteries[batt_no],
 						      "battery.charge_level.design");
 	if (bd->design_capacity < 0) {
 		bd->design_capacity = UNKNOWN;
 	}
 
 	/* last_full_capacity */
-	bd->last_full_capacity = ps_hal_get_property_int(batteries[no],
+	bd->last_full_capacity = ps_hal_get_property_int(batteries[batt_no],
 							 "battery.charge_level.last_full");
 	if (bd->design_capacity < 0) {
 		bd->last_full_capacity = UNKNOWN;
 	}
 
 	/* battery_technology */
-	bd->battery_technology = ps_hal_get_property_bool(batteries[no], "battery.is_rechargeable");
+	bd->battery_technology = ps_hal_get_property_bool(batteries[batt_no],
+							  "battery.is_rechargeable");
 	if (bd->battery_technology) {
 		bd->battery_technology = TECH_RECHARGEABLE;
 	} else if (!bd->battery_technology) {
@@ -454,43 +496,46 @@ int getBatteryDetailedInfo(const int no, BatteryDetailed * bd)
 	}
 
 	/* desing voltage */
-	bd->design_voltage = ps_hal_get_property_int(batteries[no], "battery.voltage.design");
+	bd->design_voltage = ps_hal_get_property_int(batteries[batt_no], "battery.voltage.design");
 	if (bd->design_voltage < 0) {
 		bd->design_voltage = UNKNOWN;
 	}
 
 	/* design_capacity_warning */
-	bd->design_capacity_warning = ps_hal_get_property_int(batteries[no], "battery.charge_level.warning");
+	bd->design_capacity_warning = ps_hal_get_property_int(batteries[batt_no],
+							      "battery.charge_level.warning");
 	if (bd->design_capacity_warning < 0) {
 		bd->design_capacity_warning = UNKNOWN;
 	}
 
 	/* design_capacity_low */
-	bd->design_capacity_low = ps_hal_get_property_int(batteries[no], "battery.charge_level.low");
+	bd->design_capacity_low = ps_hal_get_property_int(batteries[batt_no],
+							  "battery.charge_level.low");
 	if (bd->design_capacity_low < 0) {
 		bd->design_capacity_low = UNKNOWN;
 	}
 
 	/* capacity_granularity_1 */
-	bd->capacity_granularity_1 = ps_hal_get_property_int(batteries[no], "battery.charge_level.granularity_1");
+	bd->capacity_granularity_1 = ps_hal_get_property_int(batteries[batt_no],
+							     "battery.charge_level.granularity_1");
 	if (bd->capacity_granularity_1 < 0) {
 		bd->capacity_granularity_1 = UNKNOWN;
 	}
 
 	/* capacity_granularity_1 */
-	bd->capacity_granularity_2 = ps_hal_get_property_int(batteries[no],
+	bd->capacity_granularity_2 = ps_hal_get_property_int(batteries[batt_no],
 							     "battery.charge_level.granularity_2");
 	if (bd->capacity_granularity_2 <= 0) {
 		bd->capacity_granularity_2 = UNKNOWN;
 	}
 
 	/* alarm limit */
-	bd->alarm_limit = ps_hal_get_property_int(batteries[no], "battery.alarm.design");
+	bd->alarm_limit = ps_hal_get_property_int(batteries[batt_no], "battery.alarm.design");
 	if (bd->alarm_limit <= 0) {
 		bd->alarm_limit = UNKNOWN;
 	}
 
-	value = ps_hal_get_property_string(batteries[no], "battery.model");
+	value = ps_hal_get_property_string(batteries[batt_no], "battery.model");
 	if (value != NULL) {
 		strcpy(bd->model_number, value);
 		libhal_free_string(value);
@@ -499,7 +544,7 @@ int getBatteryDetailedInfo(const int no, BatteryDetailed * bd)
 		strcpy(bd->model_number, "UNKNOWN");
 	}
 
-	value = ps_hal_get_property_string(batteries[no], "battery.serial");
+	value = ps_hal_get_property_string(batteries[batt_no], "battery.serial");
 	if (value != NULL) {
 		strcpy(bd->serial_number, value);
 		libhal_free_string(value);
@@ -508,7 +553,7 @@ int getBatteryDetailedInfo(const int no, BatteryDetailed * bd)
 		strcpy(bd->serial_number, "UNKNOWN");
 	}
 
-	value = ps_hal_get_property_string(batteries[no], "battery.technology");
+	value = ps_hal_get_property_string(batteries[batt_no], "battery.technology");
 	if (value != NULL) {
 		strcpy(bd->battery_type, value);
 		libhal_free_string(value);
@@ -517,7 +562,7 @@ int getBatteryDetailedInfo(const int no, BatteryDetailed * bd)
 		strcpy(bd->battery_type, "UNKNOWN");
 	}
 
-	value = ps_hal_get_property_string(batteries[no], "battery.vendor");
+	value = ps_hal_get_property_string(batteries[batt_no], "battery.vendor");
 	if (value != NULL) {
 		strcpy(bd->OEM_info, value);
 		libhal_free_string(value);
@@ -541,9 +586,11 @@ int getACAdapterStatus(void)
 		return HAL_ERROR;
 	}
 
-	char **ac_adapter;
-	char *ac_device;
-	int num;
+	char **ac_adapter=NULL;
+	char *ac_device=NULL;
+	int num=0;
+	int ac_state=-1;
+	int dev_ac_state=-1;
 
 	test = checkACPI();
 	/* --------- ACPI ------------- // */
@@ -572,16 +619,28 @@ int getACAdapterStatus(void)
 	}
 	
 	if (num > 0) {
-		ac_device = strdup(*ac_adapter);
+		/* now could handle multiple ac-adapter (not just only taking the first):
+		 * will give AC_ONLINE as soon as one of the adaperts is present
+		 * will give AC_OFFLINE if at least one adapter is capable of giving status
+		 * 	and those are all not-present */
+		for (int i=0; i<num; i++)
+		{
+			ac_device = strdup(ac_adapter[i]);
+			dev_ac_state = ps_hal_get_property_bool(ac_device, "ac_adapter.present");
+			if (dev_ac_state == 1)
+				ac_state = 1;
+			else if (!dev_ac_state && ac_state == -1)
+			{
+				ac_state = 0;
+			} 
+			free(ac_device);
+		}
 		libhal_free_string_array(ac_adapter);
 	} else {
+		libhal_free_string_array(ac_adapter);
 		return NO_DEVICE_ERROR;
 	}
 	
-	int ac_state = ps_hal_get_property_bool(ac_device,
-						"ac_adapter.present");
-	free(ac_device);
-
 	if (ac_state == 1)
 		return AC_ONLINE;
 	else if (!ac_state)
@@ -633,16 +692,18 @@ int setBatteryAlarm(int percent)
 		return -1; //UNKNOWN
 	}
 
-	char **batteries;
-	int numBatteries;
+	char **batteries = NULL;
+	int num_native_batteries = 0;
+      	int num_primary_batteries = numBatteries();
 	batteries = libhal_find_device_by_capability(hal_ctx, "battery",
-						     &numBatteries, &dbus_error);
+						     &num_native_batteries, &dbus_error);
 
-	if (numBatteries == 0) {
+	if (num_primary_batteries == 0) {
+		libhal_free_string_array(batteries);
 		return -1;
 	}
 
-	BatteryDetailed *bd[numBatteries];
+	BatteryDetailed *bd[num_primary_batteries];
 
 	if (dbus_error_is_set(&dbus_error)) {
 		pDebug(DBG_INFO, "Could not get list of batteries in system: %s",
@@ -650,15 +711,15 @@ int setBatteryAlarm(int percent)
 		ps_hal_free();
 		dbus_error_free(&dbus_error);
 		libhal_free_string_array(batteries);
-		goto ERROR;
+		return UNKNOWN;
 	}
 
-	for (x = 0; x < numBatteries; x++) {
+	for (x = 0; x < num_primary_batteries; x++) {
 		bd[x] = (BatteryDetailed *) malloc(sizeof(BatteryDetailed));
 	}
 	/* iterate present battery devices. Don't iterate on existing devices: */
 	/* Imagine BAT0 -> not present, BAT1 -> present */
-	for (x = 0; (getBatteryDetailedInfo(x, bd[x]) == 0) && x < numBatteries; x++) {
+	for (x = 0; (getBatteryDetailedInfo(x, bd[x]) == 0) && x < num_primary_batteries; x++) {
 		/* I could also get general bat info again, but this would */
 		/*result in reading out battery state one more time ... */
 
@@ -689,7 +750,7 @@ int setBatteryAlarm(int percent)
 		divide_factor = (float)rem_Perc_all / percent;
 
 	pDebug(DBG_INFO, "set alarms: rem_Perc_all: %d, want alarm at: %d, divisor: %f", rem_Perc_all, percent, divide_factor);
-	for (x = 0; x < max_present_bats && x < numBatteries; x++) {
+	for (x = 0; x < max_present_bats && x < num_primary_batteries; x++) {
 		if (bd[x]->present != PRESENT_YES)
 			continue;
 		if (divide_factor < 1) {
@@ -704,22 +765,48 @@ int setBatteryAlarm(int percent)
 
 		/*******  write alarm for each battery ************************/
 		if (getDirEntry(x, dev_name, sizeof(dev_name), ACPI_BATTERY_DIR) < 0) {
-			pDebug(DBG_DIAG, "could not get acpi directory name for battery %d", x);
+			pDebug(DBG_DIAG, "could not get acpi directory name for primary battery %d", x);
 			goto ERROR;
 		}
-
-		char *path = ps_hal_get_property_string(batteries[x], "linux.acpi_path");
+		
+		char *path = NULL;
+		/* check if we need a mapping from primary to native battery-numbering */
+		if (num_primary_batteries == num_native_batteries) {
+			path = ps_hal_get_property_string(batteries[x], "linux.acpi_path");
+		} else {
+			int batt_no = -1;
+		        for (int i = 0; i < num_native_batteries; ++i) {
+		                char *value;
+		                /* check if found battery is a primary source (e.g. no ups) */
+		                value = ps_hal_get_property_string(batteries[i], "battery.type");
+		                if (value != NULL) {
+		                        if (strcmp(value, "primary") == 0) {
+		                                batt_no++;
+		                                if (batt_no == x) {
+		                                        batt_no = i;
+		                                        libhal_free_string(value);
+		                                        break;
+		                                }
+		                        }
+		                }
+		                libhal_free_string(value);
+		        }
+			path = ps_hal_get_property_string(batteries[batt_no], "linux.acpi_path");
+		}
 
 		if (path == NULL) {
 			goto ERROR;
 		}
  
-		if (strlen(path) + strlen("/alarm" + 1) > MAX_FILE_PATH) {
-			pDebug(DBG_DIAG, "Path %s/alarm too long", file);
+		if (strlen(path) + strlen("/alarm") + 1 > MAX_FILE_PATH) {
+			pDebug(DBG_DIAG, "Path %s/alarm too long", path);
+			libhal_free_string(path);
 			goto ERROR;
 		} else {
 			strcat(file, path);
 			strcat(file, "/alarm");
+			
+			libhal_free_string(path);
 
 			if (_write_line(file, "%d\n", alarm_value) < 0) {
 				pDebug(DBG_INFO, "Could not write alarm value");
@@ -729,24 +816,26 @@ int setBatteryAlarm(int percent)
 /* ##################### please review me ################################################ */
 // can this work? and what fp are we using if this fails?
 				if ((fp = fopen(file, "r")) == NULL) {
-					pDebug(DBG_DIAG, "Could not read alarm settings made");
+					pDebug(DBG_DIAG, "Could not read alarm settings made (%s)", strerror(errno));
 				}
 /* ##################### please review me ################################################ */
 
-				getColonValue(fp, verify, sizeof(verify), temp, sizeof(temp));
-				if (atoi(verify) != alarm_value) {
-					pDebug(DBG_DIAG, "failure. wrote: %d, read: %s", alarm_value, verify);
-					fclose(fp);
-					goto ERROR;
+				if (getColonValue(fp, verify, sizeof(verify), temp, sizeof(temp)) == 1) {
+					if (atoi(verify) != alarm_value) {
+						pDebug(DBG_DIAG, "failure. wrote: %d, read: %s", alarm_value, verify);
+						fclose(fp);
+						goto ERROR;
+					}
+					pDebug(DBG_DEBUG, "Alarm value %d successfully written to %s", alarm_value, file);
+				} else {
+					pDebug(DBG_INFO, "Writing of alarm value couldn't be verified");
 				}
 				fclose(fp);
-
-				pDebug(DBG_DEBUG, "Alarm value %d successfully written to %s", alarm_value, file);
 			}
 		}
 		/*******  write alarm for each battery ************************/
 	}
-	for (x = 0; x < numBatteries; x++) {
+	for (x = 0; x < num_primary_batteries; x++) {
 		if (bd[x] != NULL)
 			free(bd[x]);
 	}
@@ -754,7 +843,7 @@ int setBatteryAlarm(int percent)
 	return ret;
 
 ERROR:
-	for (x = 0; x < numBatteries; x++) {
+	for (x = 0; x < num_primary_batteries; x++) {
 		if (bd[x] != NULL)
 			free(bd[x]);
 	}
@@ -762,3 +851,4 @@ ERROR:
 	return UNKNOWN;
 
 }
+
