@@ -118,10 +118,14 @@ extern "C" {
 	extern int DEBUG_LEVEL;
 #ifdef USE_SYSLOG
 	static int syslog_open = 0;
+// FIXME: (seife, 2006-08-04)
+// pDebug does not seem to work from the library yet. Not sure if this is
+// a build issue or a general problem.
+// Because of this, it is no problem to hardcode the "powersaved" tag for now.
 #define pDebug(level, string, args...) \
 do{\
         if (!syslog_open){ \
-                openlog("[powersave]",0 , LOG_DAEMON); \
+                openlog("powersaved", LOG_PID, LOG_DAEMON); \
                 syslog_open = 1; \
         } \
         if (DEBUG_LEVEL & DBG_ERR & level){ \
@@ -266,7 +270,7 @@ do{\
 	} ThermalDev;
 	
 	/** @brief fills the given Battery struct with the current battery
-	 *  information for all found batteries
+	 *  information for all primary batteries
 	 *
 	 * supports acpi and apm
 	 *
@@ -278,9 +282,11 @@ do{\
 	 */
 	int getBatteriesInfo(BatteryGeneral * bat);
 
-	/** @brief get Information for only one battery
+	/** @brief get Information for only one primary battery
 	 *
-	 * @param no number of battery to query starting from 0
+	 * this function only returns values for primary batteries
+	 *
+	 * @param no number of primary battery to query starting from 0
 	 * @param battery_info pointer to an @ref BatteryGeneral struct
 	 * @retval 1 on success and battery is present
 	 * @retval 0 on success but battery is not present
@@ -288,13 +294,15 @@ do{\
 	 */
 	int getBatteryInfo(const int no, BatteryGeneral *battery_info);
 
-	/** @brief get number of present batteries
+	/** @brief get number of present primary batteries
 	 *
-	 * @return number batteries (also not present ones) or -1 on error
+	 * @return number of primary batteries (also not present ones) or
+	 *         -1 on error
 	 */
 	int numBatteries(void);
 
-	/** @brief use this method to get amount of batteries in the system
+	/** @brief use this method to get detailed information about a
+	 *         primary battery
 	 *
 	 * <b>ACPI only!</b>
 	 *
@@ -487,7 +495,7 @@ do{\
 	 * @retval NO_MODULE_ERROR
 	 * @retval NO_DEVICE_ERROR
 	 * @retval 0 if fan is on 
-	 * @retval -1 if fan is off
+	 * @retval 1 if fan is off
 	 * @retval -3 if fan device no. num does not exist
 	 * @retval -4 if state file does not exist 
 	 * @retval -5 if state could not be figured out
