@@ -32,6 +32,7 @@
 #include "config.h"
 
 #include "powersave_dbus.h"
+#include "powersave_hal.h"
 
 #include <string>
 #include <iostream>
@@ -83,6 +84,9 @@ static int DPM_NUMBER = 0;
 static int DPM_GET_DEVICES = 0;
 static int DPM_GET_CLASSES = 0;
 
+static int CPU_ENABLE = 0;
+static int CPU_DISABLE = 0;
+
 static int GET_CLIENTS = 0;
 
 void general_error()
@@ -138,6 +142,9 @@ void usage()
 		"   -j --dpm-device-number <x>            only suspend/resume device number <x>\n"
 		"   -J --dpm-get-devices <device class>   List devices from a specific device class\n"
 		"   -o --dpm-get-classes                  List available device classes\n\n"
+		" CPU hotplugging:\n"
+		"   -E --cpu-enable <x>                   enable a specific CPU starting from 0\n"
+		"   -D --cpu-disable <x>                  disable a specific CPU starting from 0 (CPU 0 can't be disabled)\n\n"
 		" Print information:\n"
 		"   -s, --battery-state-info               print out the current battery state\n"
 		"   -b, --battery-info                     general battery info\n"
@@ -207,6 +214,8 @@ void getArgs(int argc, char **argv)
 		{"dpm-device-number", 1, 0, 'j'},
 		{"dpm-get-devices", 1, 0, 'J'},
 		{"dpm-available-classes", 0, 0, 'o'},
+		{"cpu-enable", 1, 0, 'E'},
+		{"cpu-disable", 1, 0, 'D'},
 		{"verbose", 0, 0, 'v'},
 		{"version", 0, 0, 'V'},
 		{"clients", 0, 0, 'C'},
@@ -214,7 +223,7 @@ void getArgs(int argc, char **argv)
 		{NULL, 0, 0, 0},
 	};
 	while (1) {
-		int i = getopt_long(argc, argv, "v:p:e:crbBsSaTFVxuUmhtlfAk:KLi:I:j:J:oX:C", opts, &option_index);
+		int i = getopt_long(argc, argv, "v:p:e:crbBsSaTFVxuUmhtlfAk:KLi:I:j:J:oX:D:E:C", opts, &option_index);
 		if (argc <= 1) {
 			usage();
 			exit(0);
@@ -339,6 +348,14 @@ void getArgs(int argc, char **argv)
 		case 'o':
 			DPM_GET_CLASSES = 1;
 			break;
+		case 'E':
+			CPU_ENABLE = 1;
+			option = strtol(optarg, endptr, 10);
+			break;
+		case 'D':
+			CPU_DISABLE = 1;
+			option = strtol(optarg, endptr, 10);
+			break;
 		case 'C':
 			GET_CLIENTS = 1;
 			break;
@@ -406,13 +423,37 @@ int get_bat_info(int ext)
 
 				int ret = getBatteryInfo(x, &bg);
 
+				bat_num++;
+
 				if (ret < 0) {
+					/* be more precise
 					fprintf(stderr, "Battery error. Maybe hal is not running?\n");
 					exit(EXIT_FAILURE);
+					*/
+					switch (ret) {
+						case -3:
+							fprintf(stderr, "Internal error occured(getBattery was called with NULL-pointer).\n");
+							exit(EXIT_FAILURE);
+							break;
+						case -2:
+							fprintf(stderr, "Internal error occured(getBattery was called for notexistent bat.number).\n");
+							exit(EXIT_FAILURE);
+							break;
+						case NO_DEVICE_ERROR:
+							fprintf(stderr, "Battery %d is no primary energysource. Skipping...\n", bat_num);
+							continue;	
+							break;
+						case HAL_ERROR:
+							fprintf(stderr, "Error on hal communication. Maybe hal is not running?\n");
+							exit(EXIT_FAILURE);
+							break;
+						default:
+							fprintf(stderr, "Unknown error on getting details of Battery %d\n", bat_num);
+							exit(EXIT_FAILURE);
+							break;
+					}
 				}
 
-				bat_num++;
-
 				if (!ret) {
 					fprintf(stdout, "Battery%d: not present\n", bat_num);
 				}
@@ -911,8 +952,13 @@ int main(int argc, char **argv)
 			fprintf(stderr, "No fan devices available.\n");
 			ret = -1;
 		} else {
-			for (x = 0; x < err_code && !((current = getFanStatus(x)) < 0); x++)
-				printf("Fan no %d: %s\n", x, current ? "off" : "on");
+			//for (x = 0; x < err_code && !((current = getFanStatus(x)) < 0); x++) //stops at first fan /w error
+			for (x = 0; x < err_code; x++) {
+				if (!((current=getFanStatus(x)) < 0))
+					printf("Fan no %d: %s\n", x, current ? "off" : "on");
+				else
+					printf("Fan no %d: unknown\n",x);
+			}
 		}
 	}
 	if (REAL_CPU_SPEED) {
@@ -1138,6 +1184,58 @@ int main(int argc, char **argv)
 			ret = -1;
 		}
 	}
+	if (CPU_ENABLE) {
+		char *method = "CpufreqCPUEnable";
+
+		err_code = dbusSendMessage(ACTION_MESSAGE, method, DBUS_TYPE_INT32, &option, DBUS_TYPE_INVALID);
+
+		switch (err_code) {
+		case REPLY_HW_NOT_SUPPORTED:
+			fprintf(stderr, "CPU hotplugging not supported for this CPU.\n");
+			ret = -1;
+			break;
+		case REPLY_ALREADY_SET:
+			fprintf(stderr, "CPU already enabled.\n");
+			//noerror
+			break;
+		case REPLY_SUCCESS:
+			fprintf(stderr, "CPU %d enabled.\n", option);
+			break;
+		case REPLY_NO_RIGHTS:
+			no_connect_no_rights_error();
+			ret = -2;
+			break;
+		default:
+			general_error();
+			ret = -1;
+		}
+	}
+	if (CPU_DISABLE) {
+		char *method = "CpufreqCPUDisable";
+
+		err_code = dbusSendMessage(ACTION_MESSAGE, method, DBUS_TYPE_INT32, &option, DBUS_TYPE_INVALID);
+
+		switch (err_code) {
+		case REPLY_HW_NOT_SUPPORTED:
+			fprintf(stderr, "CPU hotplugging not supported for this CPU.\n");
+			ret = -1;
+			break;
+		case REPLY_ALREADY_SET:
+			fprintf(stderr, "CPU already disabled.\n");
+			//noerror
+			break;
+		case REPLY_SUCCESS:
+			fprintf(stderr, "CPU %d disabled.\n", option);
+			break;
+		case REPLY_NO_RIGHTS:
+			no_connect_no_rights_error();
+			ret = -2;
+			break;
+		default:
+			general_error();
+			ret = -1;
+		}
+	}
 	if (GET_CLIENTS) {
 		err_code = dbusSendSimpleMessageWithReply(REQUEST_MESSAGE, &reply,
 							  "ClientsGet");
